import {
  EntityRawSchemaArray,
  RelationDefinitionRawSchemaArray,
  Client,
  Schema,
  getLooseSchema,
  Transaction,
  TransactionQueryRef,
} from "@cubicweb/client";
import { getGet } from "./get";
import { CreateParams, CreateResult, RaRecord } from "./types";
import {
  initVariableKeyAssociation,
  VariableKeyAssociation,
} from "./utils/constants";
import {
  generateRqlCreateParts,
  makeRecursiveCreateRequest,
} from "./utils/createParams";
import { getRestrictionsClause } from "./utils/getParams";

export const getCreate = <
  E extends EntityRawSchemaArray,
  R extends RelationDefinitionRawSchemaArray<E>
>(
  client: Client,
  schema: Schema<E, R>
) => {
  const createTransactionQueries = (
    transaction: Transaction,
    resource: string,
    { data }: CreateParams
  ): { eidRef: TransactionQueryRef } => {
    const entitySchema = getLooseSchema(schema).getEntity(resource);
    if (entitySchema === null) {
      throw new RangeError(`Entity '${resource}' does not exist in the schema`);
    }

    const variableKeyAssociation: VariableKeyAssociation =
      initVariableKeyAssociation();

    const { assignments, restrictions } = generateRqlCreateParts(
      entitySchema,
      data,
      variableKeyAssociation
    );

    const requestStr = `INSERT ${resource} X: ${assignments.join(
      ", "
    )} ${getRestrictionsClause(restrictions)}`;
    return { eidRef: transaction.push(requestStr, data).ref() };
  };

  const create = async <RecordType extends RaRecord>(
    resource: string,
    { data }: CreateParams
  ): Promise<CreateResult<RecordType>> => {
    const transaction = new Transaction();
    const { eidRef } = createTransactionQueries(transaction, resource, {
      data,
    });
    const output = await client.executeTransaction(transaction);
    const eid = output.resolve(eidRef);
    return await getGet(client, schema).getOne(resource, { id: eid });
  };

  return { create, createTransactionQueries: makeRecursiveCreateRequest };
};
