const RQL_ATTRIBUTE_PREFIX = "ATR_";
const RQL_RELATION_PREFIX = "REL_";

export const RESTRICTED_PREFIX_LIST = [
  RQL_ATTRIBUTE_PREFIX,
  RQL_RELATION_PREFIX,
];

export type SelectionTransformer = (x: unknown) => unknown;

export type SortJsonType = {
  orderExpression: string;
  restriction?: string;
};

export type RelationsRqlRequestParamsType = {
  relationsTypes: string[];
  selectionTransformers: Array<SelectionTransformer>;
  selection: string[];
  groupBy: string[];
  restrictions: string[];
  sortVariable: string | null;
};

export interface VariableKeyAssociation {
  _lastVariableAttributeId: number;
  _lastVariableRelationId: number;
  variables: { [key: string]: string };
}

const _nextVar = (
  variableKeyAssociation: VariableKeyAssociation,
  mode: "attribute" | "relation"
): string => {
  let newVar = "";
  if (mode === "attribute") {
    variableKeyAssociation._lastVariableAttributeId++;
    newVar = `ATTR_${variableKeyAssociation._lastVariableAttributeId}`;
  } else {
    variableKeyAssociation._lastVariableRelationId++;
    newVar = `REL_${variableKeyAssociation._lastVariableRelationId}`;
  }
  return newVar;
};

/**
 * Generates a RQL variable from the given relation key
 *
 * @param relationKey The relation definition key
 * @param mode Is the variable an attribute or a relaiton
 * @returns A RQL variable for the given relation
 */
export const getRelationVariable = (
  relationKey: string,
  mode: "attribute" | "relation",
  variableKeyAssociation: VariableKeyAssociation
) => {
  if (Object.keys(variableKeyAssociation.variables).includes(relationKey)) {
    return variableKeyAssociation.variables[relationKey];
  }
  const newVar = _nextVar(variableKeyAssociation, mode);
  variableKeyAssociation.variables[relationKey] = newVar;
  return newVar;
};

export const initVariableKeyAssociation = (): VariableKeyAssociation => ({
  _lastVariableAttributeId: 0,
  _lastVariableRelationId: 0,
  variables: {},
});
